MODULE TestSPF;

        (********************************************************)
        (*                                                      *)
        (*      Extracting SPF information from nameserver.     *)
        (*                                                      *)
        (*  Programmer:         P. Moylan                       *)
        (*  Started:            03 October 2016                 *)
        (*  Last edited:        22 October 2016                 *)
        (*  Status:             OK                              *)
        (*                                                      *)
        (********************************************************)


IMPORT TextIO, Strings;

FROM SYSTEM IMPORT
    (* type *)  CARD8, CARD32,
    (* proc *)  CAST;

FROM SPF IMPORT
    (* type *)  SPFresult,
    (* proc *)  DoSPFLookup, WriteSPFresult;

FROM ProgramArgs IMPORT
    (* proc *)  ArgChan, IsArgPresent;

FROM STextIO IMPORT
    (* proc *)  WriteChar, WriteString, WriteLn;

FROM IOChan IMPORT
    (* type *)  ChanId;

FROM Inet2Misc IMPORT
    (* proc *)  IPToString, StringToIP;

FROM Names IMPORT
    (* type *)  DomainName;

(************************************************************************)

TYPE FourByte = ARRAY [0..3] OF CARD8;

CONST
    Nul = CHR(0);
    DefaultAddr = FourByte {192, 64, 237, 130};
    DefaultDomain = "aol.com";

(************************************************************************)
(*                      GET THE PROGRAM ARGUMENTS                       *)
(************************************************************************)

PROCEDURE GetParams (VAR (*OUT*) IPaddr: CARDINAL;
                         VAR (*OUT*) domain: DomainName);

    VAR args: ChanId;  pos: CARDINAL;  found: BOOLEAN;
        ParameterString, IPstring: ARRAY [0..79] OF CHAR;

    BEGIN
        IPaddr := 0;
        domain := "";
        args := ArgChan();
        IF IsArgPresent() THEN
            TextIO.ReadString (args, ParameterString);
        ELSE
            ParameterString := "";
        END (*IF*);
        IPstring := ParameterString;
        Strings.FindNext (' ', IPstring, 0, found, pos);
        IF found THEN
            IPstring[pos] := Nul;
            IPaddr := StringToIP (IPstring);
            Strings.Delete (ParameterString, 0, pos+1);
        END (*IF*);
        Strings.Assign (ParameterString, domain);
        pos := LENGTH (domain);
        IF pos > 0 THEN
            DEC (pos);
            WHILE domain[pos] = ' ' DO
                domain[pos] := Nul;  DEC(pos);
            END (*WHILE*);
        END (*IF*);

        (* Use default values if necessary. *)

        IF IPaddr = 0 THEN IPaddr := CAST(CARD32, DefaultAddr); END (*IF*);
        IF domain[0] = Nul THEN domain := DefaultDomain; END (*IF*);

        (* Echo the parameters back to the user. *)

        IPToString (IPaddr, FALSE, IPstring);
        WriteString ("IP address = '");  WriteString (IPstring);
        WriteChar ("'");  WriteLn;
        WriteString ("domain = '");  WriteString (domain);
        WriteChar ("'");  WriteLn;

    END GetParams;

(************************************************************************)
(*                       TESTING THE SPF MODULE                         *)
(************************************************************************)

PROCEDURE CheckSPFRecord (ipaddr: CARDINAL;  domain: DomainName;
                                                sender: ARRAY OF CHAR);

    (* Does an SPF query for the given address and domain.  *)

    CONST BufferSize = 1023;

    TYPE BufferSubscript = [0..BufferSize-1];

    VAR SPFstring: ARRAY BufferSubscript OF CHAR;
        result: SPFresult;

    BEGIN
        result := DoSPFLookup (ipaddr, domain, sender, SPFstring);

        IF SPFstring[0] = Nul THEN
            WriteString ("No SPF record found");
        ELSE
            WriteString ("SPF record: ");  WriteString (SPFstring);
        END (*IF*);
        WriteLn;
        WriteString ("SPF result: ");  WriteSPFresult (result);  WriteLn;

    END CheckSPFRecord;

(************************************************************************)

PROCEDURE DoSPFTest;

    VAR addr: CARDINAL;
        domain: DomainName;

    BEGIN
        GetParams (addr, domain);
        CheckSPFRecord (addr, domain, "");
    END DoSPFTest;

(************************************************************************)

BEGIN
    DoSPFTest;
END TestSPF.

